@@ -6,6 +6,10 @@ import os
 import sqlite3
 import pickle
 
+from itertools import chain, repeat
+
+import imp
+
 try:
 	import psycopg2
 except ImportError:
@@ -18,6 +22,11 @@ except ImportError:
 
 logdir = os.path.expanduser("~")
 logdir = os.path.join(logdir, ".tech-parser")
+try:
+	config = imp.load_source('config',
+		os.path.join(logdir, 'user_parser_config.py'))
+except ImportError:
+	from TechParser import parser_config as config
 
 r1 = re.compile(r"(?P<g1>\w+)n['\u2019]t", re.UNICODE)
 r2 = re.compile(r"(?P<g1>\w+)['\u2019]s", re.UNICODE)
@@ -29,6 +38,27 @@ r7 = re.compile(r"(?P<g1>\w+)['\u2019]ll", re.UNICODE)
 r8 = re.compile(r"gonna", re.UNICODE)
 r9 = re.compile(r"\W", re.UNICODE)
 
+class OuterZipStopIteration(Exception):
+	pass
+
+def outer_zip(*args):
+	count = [len(args) - 1]
+
+	def sentinel(default):
+		if not count[0]:
+			raise OuterZipStopIteration
+		count[0] -= 1
+		yield default
+
+	iters = [chain(p, sentinel(default), repeat(default)) for p, default in args]
+	result = []
+	
+	try:
+		while iters:
+			result.append(tuple(map(next, iters)))
+	except OuterZipStopIteration:
+		return result
+
 def get_similarity(pairs1, pairs2):
 	len_all_pairs = len(pairs1) + len(pairs2)
 	shrd = pairs1 & pairs2
@@ -55,9 +85,33 @@ def pairs_fromstring(s):
 def article_pairs(a):
 	return pairs_fromstring(a['title']), pairs_fromstring(a['summary'])
 
+def get_word_pairs(words):
+	pairs = []
+	
+	for i in words:
+		if isinstance(i, tuple):
+			word = i[0]
+			priority = i[1]
+		else:
+			word = i
+			priority = 1.0
+		
+		pairs.append((pairs_fromstring(word), priority))
+	
+	return pairs
+
 def find_similiar(articles, db='sqlite'):
-	interesting_articles = get_interesting_articles(db)
-	blacklist = get_blacklist(db)
+	con = connect_db(db)
+	cur = con.cursor()
+	
+	interesting_articles = get_interesting_articles(db, cur)
+	blacklist = get_blacklist(db, cur)
+	
+	con.close()
+	
+	interesting_word_pairs = get_word_pairs(config.interesting_words)
+	boring_word_pairs = get_word_pairs(config.boring_words)
+	
 	ignored_links = [i['link'] for i in blacklist]
 	processed, scores = [], []
 	interesting_links = [i['link']
@@ -68,12 +122,8 @@ def find_similiar(articles, db='sqlite'):
 	ignored_pairs = [article_pairs(i)
 		for i in blacklist[:150]]
 	
-	while len(interesting_pairs) > len(ignored_pairs):
-		ignored_pairs.append(set())
-	while len(ignored_pairs) > len(interesting_pairs):
-		interesting_pairs.append(set())
-	
-	zipped = list(zip(interesting_pairs, ignored_pairs))
+	zipped = outer_zip((interesting_pairs, set()), (ignored_pairs, set()),
+		(interesting_word_pairs, tuple()), (boring_word_pairs, tuple()))
 	
 	for article in articles:
 		if article['link'] in interesting_links or \
@@ -84,11 +134,15 @@ def find_similiar(articles, db='sqlite'):
 		
 		pairs1 = article_pairs(article)
 		
-		for (pairs2, pairs3) in zipped:
+		for (pairs2, pairs3, pairs4, pairs5) in zipped:
 			if pairs2:
 				score += get_similarity2(pairs1, pairs2)
 			if pairs3:
 				score -= get_similarity2(pairs1, pairs3)
+			if pairs4:
+				score += get_similarity2(pairs1, [pairs4[0], set()])*pairs4[1]
+			if pairs5:
+				score -= get_similarity2(pairs1, [pairs5[0], set()])*pairs5[1]
 		
 		processed.append(article)
 		scores.append(score)
@@ -117,15 +171,22 @@ def get_pairs(words):
 		pairs.update({word[i:i+2] for i in range(len(word))})
 	return pairs
 
-def get_interesting_articles(db='sqlite'):
-	setup_db(db)
-	connect, args, kwargs = which_db(db)
-	con = connect(*args, **kwargs)
-	cur = con.cursor()
+def get_interesting_articles(db='sqlite', cur=None):
+	if cur is None:
+		con = connect_db(db)
+		cur = con.cursor()
+	
+	setup_db(db, cur)
+	
 	cur.execute('''SELECT title, link, summary, fromrss, icon, color, source
 		FROM interesting_articles ORDER BY id DESC;''')
 	res = cur.fetchall()
-	con.close()
+	
+	try:
+		con.close()
+	except NameError:
+		pass
+	
 	return [{'title': x[0],
 			'link': x[1],
 			'summary': x[2],
@@ -134,15 +195,22 @@ def get_interesting_articles(db='sqlite'):
 			'color': x[5],
 			'source': x[6]} for x in res]
 
-def get_blacklist(db='sqlite'):
-	setup_db(db)
-	connect, args, kwargs = which_db(db)
-	con = connect(*args, **kwargs)
-	cur = con.cursor()
+def get_blacklist(db='sqlite', cur=None):
+	if cur is None:
+		con = connect_db(db)
+		cur = con.cursor()
+	
+	setup_db(db, cur)
+	
 	cur.execute('''SELECT title, link, summary, fromrss, icon, color, source
 		FROM blacklist ORDER BY id DESC;''')
 	res = cur.fetchall()
-	con.close()
+	
+	try:
+		con.close()
+	except NameError:
+		pass
+	
 	return [{'title': x[0],
 			'link': x[1],
 			'summary': x[2],
@@ -151,7 +219,7 @@ def get_blacklist(db='sqlite'):
 			'color': x[5],
 			'source': x[6]} for x in res]
 
-def add_article(addr, db='sqlite'):
+def add_article(addr, db='sqlite', cur=None):
 	f = open(os.path.join(logdir, "articles_dumped"), 'rb')
 	dumped = f.read()
 	f.close()
@@ -160,14 +228,10 @@ def add_article(addr, db='sqlite'):
 	
 	for article in articles:
 		if article[0]['link'] == addr:
-			add_to_interesting(article, db)
+			add_to_interesting(article, db, cur)
 			break
 
-def add_to_interesting(article, db='sqlite'):
-	setup_db(db)
-	
-	connect, args, kwargs = which_db(db)
-	
+def add_to_interesting(article, db='sqlite', cur=None):	
 	if db == 'sqlite':
 		IntegrityError = sqlite3.IntegrityError
 	else:
@@ -175,9 +239,12 @@ def add_to_interesting(article, db='sqlite'):
 			IntegrityError = psycopg2.IntegrityError
 		except NameError:
 			IntegrityError = sqlite3.IntegrityError
+	if cur is None:
+		con = connect_db(db)
+		cur = con.cursor()
+	
+	setup_db(db, cur)
 	
-	con = connect(*args, **kwargs)
-	cur = con.cursor()
 	cur.execute('SELECT count(link) FROM interesting_articles;')
 	if cur.fetchone()[0] > 1000:
 		cur.execute("""DELETE FROM interesting_articles
@@ -204,13 +271,16 @@ def add_to_interesting(article, db='sqlite'):
 		color = article[0].get('color', '#000')
 		
 		cur.execute(code, (title, link, summary, fromrss, icon, color, source))
-		con.commit()
 	except IntegrityError:
 		pass
 	
-	con.close()
+	try:
+		con.commit()
+		con.close()
+	except NameError:
+		pass
 
-def add_article_to_blacklist(addr, db='sqlite'):
+def add_article_to_blacklist(addr, db='sqlite', cur=None):
 	f = open(os.path.join(logdir, "articles_dumped"), 'rb')
 	dumped = f.read()
 	f.close()
@@ -220,17 +290,13 @@ def add_article_to_blacklist(addr, db='sqlite'):
 	for article in articles:
 		if article[0]['link'] == addr:
 			f = open(os.path.join(logdir, "articles_dumped"), 'wb')
-			add_to_blacklist(article, db)
+			add_to_blacklist(article, db, cur)
 			articles.remove(article)
 			f.write(pickle.dumps(articles))
 			f.close()
 			break
 
-def add_to_blacklist(article, db='sqlite'):
-	setup_db(db)
-	
-	connect, args, kwargs = which_db(db)
-	
+def add_to_blacklist(article, db='sqlite', cur=None):
 	if db == 'sqlite':
 		IntegrityError = sqlite3.IntegrityError
 	else:
@@ -239,15 +305,18 @@ def add_to_blacklist(article, db='sqlite'):
 		except NameError:
 			IntegrityError = sqlite3.IntegrityError
 	
-	con = connect(*args, **kwargs)
-	cur = con.cursor()
+	if cur is None:
+		con = connect_db(db)
+		cur = con.cursor()
+	
+	setup_db(db, cur)
 
-	sqlite_code = """INSERT INTO
+	sqlite_code = '''INSERT INTO
 			blacklist(title, link, summary, fromrss, icon, color, source)
-				VALUES(?, ?, ?, ?, ?, ?, ?);"""
-	postgres_code = """INSERT INTO
+				VALUES(?, ?, ?, ?, ?, ?, ?);'''
+	postgres_code = '''INSERT INTO
 			blacklist(title, link, summary, fromrss, icon, color, source)
-				VALUES(%s, %s, %s, %s, %s, %s, %s);"""
+				VALUES(%s, %s, %s, %s, %s, %s, %s);'''
 	
 	if db == 'sqlite':
 		code = sqlite_code
@@ -263,19 +332,21 @@ def add_to_blacklist(article, db='sqlite'):
 		icon = article[0].get('icon', '')
 		color = article[0].get('color', '#000')
 		cur.execute(code, (title, link, summary, fromrss, icon, color, source))
-		con.commit()
 	except IntegrityError:
 		pass
 	
-	con.close()
+	try:
+		con.commit()
+		con.close()
+	except NameError:
+		pass
 
-def remove_from_history(link, db='sqlite'):
-	setup_db(db)
+def remove_from_history(link, db='sqlite', cur=None):
+	if cur is None:
+		con = connect_db(db)
+		cur = con.cursor()
 	
-	connect, args, kwargs = which_db(db)
-	
-	con = connect(*args, **kwargs)
-	cur = con.cursor()
+	setup_db(db, cur)
 	
 	sqlite_code = 'DELETE FROM interesting_articles WHERE link=?;'
 	postgres_code = 'DELETE FROM interesting_articles WHERE link=%s;'
@@ -283,16 +354,19 @@ def remove_from_history(link, db='sqlite'):
 	code = postgres_code if db == 'postgresql' else sqlite_code
 	
 	cur.execute(code, (link,))
-	con.commit()
-	con.close()
+	
+	try:
+		con.commit()
+		con.close()
+	except NameError:
+		pass
 
-def remove_from_blacklist(link, db='sqlite'):
-	setup_db(db)
+def remove_from_blacklist(link, db='sqlite', cur=None):
+	if cur is None:
+		con = connect_db(db)
+		cur = con.cursor()
 	
-	connect, args, kwargs = which_db(db)
-	
-	con = connect(*args, **kwargs)
-	cur = con.cursor()
+	setup_db(db, cur)
 	
 	sqlite_code = 'DELETE FROM blacklist WHERE link=?;'
 	postgres_code = 'DELETE FROM blacklist WHERE link=%s;'
@@ -300,15 +374,18 @@ def remove_from_blacklist(link, db='sqlite'):
 	code = postgres_code if db == 'postgresql' else sqlite_code
 	
 	cur.execute(code, (link,))
-	con.commit()
-	con.close()
+	try:
+		con.commit()
+		con.close()
+	except NameError:
+		pass
 
-def setup_db(db='sqlite'):
-	connect, args, kwargs = which_db(db)
+def setup_db(db='sqlite', cur=None):
+	if cur is None:
+		con = connect_db(db)
+		cur = con.cursor()
 	
-	con = connect(*args, **kwargs)
-	cur = con.cursor()
-	sqlite_code = """CREATE TABLE IF NOT EXISTS interesting_articles
+	sqlite_code = '''CREATE TABLE IF NOT EXISTS interesting_articles
 			(id INTEGER PRIMARY KEY AUTOINCREMENT,
 				title TEXT, link TEXT, summary TEXT, source TEXT,
 				fromrss INTEGER, icon TEXT, color TEXT,
@@ -317,42 +394,42 @@ def setup_db(db='sqlite'):
 			(id INTEGER PRIMARY KEY AUTOINCREMENT,
 				fromrss INTEGER, icon TEXT, color TEXT,
 				title TEXT, link TEXT, summary TEXT, source TEXT,
-				UNIQUE(link))"""
-	postgres_code = """CREATE TABLE IF NOT EXISTS interesting_articles
+				UNIQUE(link))'''
+	postgres_code = '''CREATE TABLE IF NOT EXISTS interesting_articles
 			(id SERIAL,	title TEXT, link TEXT, summary TEXT,
 				fromrss INTEGER, icon TEXT, color TEXT,
 				source TEXT, UNIQUE(link));
 		CREATE TABLE IF NOT EXISTS blacklist
 			(id SERIAL, title TEXT, link TEXT, summary TEXT,
 				fromrss INTEGER, icon TEXT, color TEXT,
-				source TEXT, UNIQUE(link))"""
+				source TEXT, UNIQUE(link))'''
 	if db == 'sqlite':
 		code = sqlite_code
 	elif db == 'postgresql':
 		code = postgres_code
 	for statement in code.split(';'):
 		cur.execute(statement)
-	con.commit()
-	con.close()
+	
+	try:
+		con.commit()
+		con.close()
+	except NameError:
+		pass
 
 def which_db(db):
 	if db == 'postgresql':
 		try:
-			connect = psycopg2.connect
-			kwargs = parse_dburl()
-			args = tuple()
+			connect = lambda: psycopg2.connect(**parse_dburl())
 		except NameError:
-			connect = sqlite3.connect
-			kwargs = {}
-			args = (os.path.join(logdir, 'interesting.db'),)
+			arg = os.path.join(logdir, 'interesting.db')
+			connect = lambda: sqlite3.connect(arg)
 	elif db == 'sqlite':
-		connect = sqlite3.connect
-		kwargs = {}
-		args = (os.path.join(logdir, 'interesting.db'),)
+		arg = os.path.join(logdir, 'interesting.db')
+		connect = lambda: sqlite3.connect(arg)
 	else:
 		raise Exception('db must be sqlite or postgresql')
 	
-	return (connect, args, kwargs)
+	return connect
 
 def parse_dburl(var='DATABASE_URL'):
 	parsed = urlparse(os.environ.get(var, ''))
@@ -362,3 +439,6 @@ def parse_dburl(var='DATABASE_URL'):
 			'host': parsed.hostname,
 			'port': parsed.port,
 			'dbname': parsed.path[1:]}
+
+def connect_db(db='sqlite'):
+	return which_db(db)()
